package iterator;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

/**
 * How to traverse collections (implicit or explicit use of iterator)
 *
 * @author hajo
 *
 */
public class TraverseCollections {

	// List and Set implements Collection which implements
    // iterable
    public static void traverse(List<Date> list) {
        // Can use short for loop if iterable
        for (Date d : list) {
            System.out.println(d);
        }
    }

    public static void traverse(Set<Date> set) {
        for (Date d : set) {
            System.out.println(d);
        }
    }

    // Map is special
    public static void traverse(Map<String, Date> map) {
        for (Entry<String, Date> e : map.entrySet()) {
            System.out.println(e.getKey() + e.getValue());
        }
    }

    // Get the iterator explicitly
    public static void traverseIterator(List<Date> list) {
        Iterator<Date> i = list.iterator();
        while (i.hasNext()) {
            System.out.println(i.next());
        }
    }

    public static void traverseAndModify(List<Integer> list) {
        for (Integer i : list) {
            if (i == 44) {
                list.remove(i); // Bad must use iterator
            }
        }
    }

    //Use iterator to remove
    public static void removeUsingIterator(List<Integer> list) {
        Iterator<Integer> i = list.iterator();
        while (i.hasNext()) {
            int ii = i.next();
            if (ii == 44) {
                i.remove(); // Ok
            }
        }
    }

    public static void main(String[] args) {
        traverse(new ArrayList<Date>());
        traverse(new HashMap<String, Date>());
        traverse(new HashSet<Date>());
        traverseIterator(new ArrayList<Date>());

        List<Integer> il = new ArrayList<Integer>();
        for (int i = 0; i < 100; i++) {
            il.add(i);
        }
        //traverseAndModify(il); 
        removeUsingIterator(il);
    }
}
