package canonical.clone;

import java.util.Date;

/**
 * Basic implementation of clone see also inheritance
 */
// Must implement marker interface Cloneable
public class ClassToClone implements Cloneable {

	private int i = 111;
	// clone is incompatible with normal use of
	// final fields for mutable objects
	private/* final won't work */ Date date = new Date();

	/*
	 * If other non-subclasses should be able to clone,
	 * must override and make public (clone is protected in Object)
	 */
	@Override
	public Object clone() {
		// This class direct ancestor or Object!!
		// If exception the "system" really has a problem, so...
		try {
			// The standard idiom. Constructor NOT allowed
		    ClassToClone a = (ClassToClone) super.clone();
		    a.date = new Date(date.getTime());
			return a;  
		} catch (CloneNotSupportedException e) {
			// .. something is very wrong
			throw new InternalError();
		}
	}
	
	// If using the below we can't clone this or any subclass
	/*
	 * public Object clone() throws CloneNotSupportedException { 
	 * throw new CloneNotSupportedException("Can't clone " + this); 
	 * }
	 */
	public int getI() {
        return i;
    }
    
    public void setI(int i) {
        this.i = i;
    }
      
    public Date getDate() {
        return date;
    }

    //----- Equals and hashCode to be used in Main ---------
    
    
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((date == null) ? 0 : date.hashCode());
		result = prime * result + i;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ClassToClone other = (ClassToClone) obj;
		if (date == null) {
			if (other.date != null)
				return false;
		} else if (!date.equals(other.date))
			return false;
		if (i != other.i)
			return false;
		return true;
	}
    
    
    
    
}
