package dp.proxy;

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Proxy pattern
 * A proxy is a wrapper (with same interface) adding
 * something to the wrapped object
 * 
 * @author hajo
 *
 */
public class UsingAProxy {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		IA a = new A();
		IA ap = new LoggingProxy(a);

		System.out.println(ap.doIt());
	}
	
	//------- TYPES------------
	
	public interface IA {
		public int doIt();
	}

	// No logging service in A (SRP)
	public static class A implements IA{
		public int doIt(){
			return 42;
		}
	}
	
	// Proxy adds logging facilities to A
	public static class LoggingProxy implements IA{
		private final IA a;
		
		public LoggingProxy( IA a){
			this.a = a;
		}
		
		public int doIt(){
			Logger.getAnonymousLogger().log(Level.FINE, "Loggin something");
			return a.doIt();
		}
	}
}
