package exception.bestpractices;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

/**
 * If a method throws all exceptions, then it may use a finally with no catch
 *
 * @author Java Best Practices
 *
 */
public class FileHandler {

    public void simpleFinally(String aFileName) throws IOException {
	//If this line throws an exception, then neither the try block
        //nor the finally block will execute.
        //That is a good thing, since reader would be null.
        BufferedReader reader = new BufferedReader(new FileReader(aFileName));
        try {
            //Any exception in the try block will cause the finally block to execute
            String line = null;
            while ((line = reader.readLine()) != null) {
                //process the line...
            }
        } finally {
            //The reader object will never be null here.
            //This finally is only entered after the try block is 
            //entered. But, it's NOT POSSIBLE to enter the try block 
            //with a null reader object.
            reader.close();
        }
    }
}
