package mvc;

import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.WindowConstants;
import mvc.eventbus.Event;
import mvc.eventbus.EventBus;
import mvc.eventbus.IEventHandler;

/**
 * The view part of application
 *
 * @author hajo
 *
 */
@SuppressWarnings("serial")
public class MainFrame extends JFrame implements IEventHandler,
        ActionListener {

    private final Control ctrl;

    public MainFrame(Control control) {
        init();
        this.ctrl = control;
        // This is better done in some factory, just for now
        EventBus.INSTANCE.register(this);
    }

    /**
     * **********************************
     * A Swing listener
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == btn) {
            String s = in.getText();
            // Call control 
            ctrl.execute(Integer.valueOf(s));
            // Also possible to have reference to model
        }
    }

    /**
     * ************************************
     * Updates from model, using Observer/EventBus
     */
    @Override
    public void onEvent(Event evt) {
        if (evt.getTag() == Event.Tag.I_CHANGED) {
            int i = (int) evt.getValue();
            out.setText(String.valueOf(i));
        }

    }

    // --------- Boring GUI code below --------------
    JButton btn = new JButton("SetI (integer)");
    JTextField in = new JTextField("");
    JLabel out = new JLabel();

    private void init() {
        this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        this.setLayout(new GridLayout(3, 1));
        Container c = this.getContentPane();
        c.add(in);
        btn.addActionListener(this);
        c.add(btn);
        c.add(out);
        // pack(); Use pack OR setSize
        this.setSize(200, 200);
    }

}
