package dp.iterator;

import java.util.Arrays;
import java.util.ConcurrentModificationException;
import java.util.Iterator;
import java.util.NoSuchElementException;

// Class do have an iterator (iterable)
// Also be used in foreach loop (check)

  // CHECK CONCURRENCY see java Colletions javadoc API, synchronizedCollection
// http://download.oracle.com/javase/1.4.2/docs/api/java/util/Collections.html#synchronizedMap%28java.util.Map%29
public class Container implements IContainer<Integer> {

	private static int DEFAULT_SIZE = 1024;
	private int mods = 0;
	private Integer[] elems = new Integer[DEFAULT_SIZE];

	@Override
	public Iterator<Integer> iterator() {
		return new FailFastIter();
		// return new NoFailIter();
		// return null;
	}

	private class FailFastIter implements Iterator<Integer> {
		int currentIndex = 0;
		int check = -1;
		int size = elems.length;

		public FailFastIter() {
			check = mods;
		}

		@Override
		public boolean hasNext() {
			checkForModification();
			if (currentIndex < size) {
				return true;
			}
			return false;
		}

		@Override
		public Integer next() {
			checkForModification();
			Integer result = elems[currentIndex];
			currentIndex++;
			return result;
		}

		@Override
		public void remove() { // TODO
			throw new UnsupportedOperationException();
		}

		private void checkForModification() {
			if (check != mods) {
				throw new ConcurrentModificationException();
			}
		}
	}

	//-----------------------------------------------------
	
	private class FailSafeIter implements Iterator<Integer>{
	    int currentIndex = 0;
	    // Make a copy of the data (so no concurrent modification)
	    Integer[] internal = Arrays.copyOf(elems, size());
	    int internalSize = -1;
	 
	    public FailSafeIter(){
	        internalSize = internal.length;
	    }
	 
	    @Override
	    public boolean hasNext(){
	        if (currentIndex < internalSize){
	            return true;
	        }
	        return false;
	    }
	 
	    @Override
	    public Integer next(){
	        if (hasNext()){
	            Integer result = internal[currentIndex];
	            currentIndex++;
	            return result;
	        }else{
	            throw new NoSuchElementException();
	        }
	    }
	 
	    @Override
	    public void remove(){
	        throw new UnsupportedOperationException();
	    }
	}
	@Override
	public Integer getElement(int position) {
		if (position <= 0 && position < DEFAULT_SIZE) {
			return elems[position];
		}
		throw new IllegalArgumentException("Bad position");
	}

	@Override
	public void setElement(int position, Integer t) {
		if (position <= 0 && position < DEFAULT_SIZE) {
			elems[position] = t;
			 mods++;     // Track changes !!!
		}
		throw new IllegalArgumentException("Bad position");
	}

	@Override
	public int size() {
		return elems.length;

	}
}
