package dp.visitor;

/**
 * Using the Visitor pattern. For a variation see visitor.wildcards
 * @author hajo
 *
 */
public class Main {

    public static void main(String[] args) {
        Node<A> tree;
        tree = new Node<>(new A(1), new Node<>(new A(1), new Leaf<>(
                new A(1)), new Leaf<>(new A(1))), new Node<>(new A(1),
                        new Leaf<>(new A(1)), new Leaf<>(new A(1))));

        SumVisitor sv = new SumVisitor();

        Double d = sv.visit(tree);
        System.out.println(d);

        // Can't use this here, see visitor.wildcards
        Node<B> tree2;
        tree2 = new Node<>(new B(1), new Node<>(new B(1),
                new Leaf<>(new B(1)), new Leaf<>(new B(1))), new Node<>(
                        new B(1), new Leaf<>(new B(1)), new Leaf<>(new B(1))));

        // Tree with sub class data *NOT* accepted
        // Double d2 = sv.visit(tree2);

        StringVisitor strv = new StringVisitor();
        String s = strv.visit(tree);
        System.out.println(s);

        // Tree with sub class *NOT* data accepted
        // String s2 = strv.visit(tree2);
        // System.out.println(s2);

    }

    // ---------- Two different Visitors ---------

    public static class SumVisitor implements IVisitor<Double, A> {

        @Override
        public Double visit(Leaf<A> leaf) {
            return 1.0 * leaf.getIt().i;
        }

        @Override
        public Double visit(Node<A> node) {
            return 1.0 * node.getIt().i + node.getLeft().accept(this)
                    + node.getRight().accept(this);
        }
    }

    public static class StringVisitor implements IVisitor<String, A> {

        @Override
        public String visit(Leaf<A> leaf) {
            return "Leaf:" + leaf.getIt();
        }

        @Override
        public String visit(Node<A> node) {
            return "Node:" + node.getIt() + ",(" + node.getLeft().accept(this)
                    + "), (" + node.getRight().accept(this) + ")";
        }
    }

    // ------------- Just Types -------------------------------

    public static class A {
        public final int i;

        public A(int i) {
            this.i = i;
        }

        @Override
        public String toString() {
            return "A";
        }
    }

    public static class B extends A {
        public B(int i) {
            super(i);
        }

        @Override
        public String toString() {
            return "B";
        }
    }
}
