package dp.visitor.wildcards;

/**
 * Using the Visitor pattern
 * @author hajo
 *
 */
public class Main {

    public static void main(String[] args) {
        Node<A> tree = new Node<>(new A(1), new Node<>(new A(1), new Leaf<>(
                new A(1)), new Leaf<>(new A(1))), new Node<>(new A(1),
                new Leaf<>(new A(1)), new Leaf<>(new A(1))));

        SumVisitor sv = new SumVisitor();

        Double d = sv.visit(tree);
        System.out.println(d);

        Node<B> tree2 = new Node<>(new B(1), new Node<>(new B(1),
                new Leaf<>(new B(1)), new Leaf<>(new B(1))), new Node<>(
                new B(1), new Leaf<>(new B(1)), new Leaf<>(new B(1))));

        // Tree with sub class data accepted
        Double d2 = sv.visit(tree2);

        StringVisitor strv = new StringVisitor();
        String s = strv.visit(tree);
        System.out.println(s);

        // Tree with sub class data accepted
        String s2 = strv.visit(tree2);
        System.out.println(s2);
    }

    // ---------- Two different Visitors ---------

    public static class SumVisitor implements IVisitor<Double, A> { // Can't have ? extands A here

        @Override
        public Double visit(Leaf<? extends A> leaf) {
            return 1.0 * leaf.getIt().i;
        }

        @Override
        public Double visit(Node<? extends A> node) {
            // Get this
            // ITree<? extends A> t = node.getLeft();
            // Can't
            // ITree<A> tt = t;
            /*
               Must have :  IVisitor<Double, ? extends A> v)
               for call to accept(..) but "this" is IVisitor<Double, A>!
             */  
            // Solved by using ? super T in accept 
            // Now we get <Double, ? super ? extends A> == <Double, A> (?)
            return 1.0 * node.getIt().i + node.getLeft().accept(this)
                    + node.getRight().accept(this);
        }
    }

    public static class StringVisitor implements IVisitor<String, A> {

        @Override
        public String visit(Leaf<? extends A> leaf) {
            return "Leaf:" + leaf.getIt();
        }

        @Override
        public String visit(Node<? extends A> node) {
            return "Node:" + node.getIt() + ",(" + node.getLeft().accept(this)
                    + "), (" + node.getRight().accept(this) + ")";
        }
    }

    // ------------- Just Types -------------------------------

    public static class A {
        public final int i;

        public A(int i) {
            this.i = i;
        }

        @Override
        public String toString() {
            return "A";
        }
    }

    public static class B extends A {
        public B(int i) {
            super(i);
        }

        @Override
        public String toString() {
            return "B";
        }
    }
}
