package generic.erasure.consequences;

import java.util.ArrayList;
import java.util.List;

/**
 * Type erasure limits the programmer severely (can probably 
 * solve using reflection)
 * 
 * @author hajo
 *
 */
public class TypeErsureConsequencesForProgrammer {

	// ---------- 1 (numbering from slides) ----------

	/* No! E gone at compile
	public static <E> void append(List<E> list) {
	    E elem = new E();  // compile-time error
	    list.add(elem);
	    
	    E[] elems = new E[3];  No
	}*/

	/*
	 * As a workaround, you can create an object of a type parameter 
	 * through reflection, see reflection
	 */
	public static <E> void append(List<E> list, Class<E> cls) throws Exception {
		E elem = cls.newInstance(); // OK
		list.add(elem);
	}

	// ---------- 2 ----------------------

	/*
	 * Static shared by all non-static objects of the class. 
	 * Static fields of type parameters are not allowed. 
	public class MobileDevice<T> {
	    private static T os;

	    
	}*/

	// ---------- 3 ----------------------
	/* No! Integer gone
	public static <E> void rtti(List<E> list) {
	    if (list instanceof ArrayList<Integer>) {  // compile-time error
	        // ...
	    }
	}*/

	public static void rtti(List<?> list) {
		if (list instanceof ArrayList<?>) { // OK; instanceof requires a reifiable type
			// ...
		}
	}

	/*  No! Cannot cast to a parameterized type
	{
		List<Integer> li = new ArrayList<>();
		List<Number> ln = (List<Number>) li; // compile-time error
	}*/

	{ // But sometimes works... (compiler knows..)
		List<String> l1 = new ArrayList<>();
		List<String> l2 = (List<String>) l1; // OK
	}
	
	public static <T> T[] castIt(Object[] list) {
		// Warning, compiler can't check (T gone)
		return (T[]) list;
	}
	
	
	// ---------- 4 ----------------------
	//You cannot create arrays of parameterized types.
	/*
	public static <T> T[] createArray( T t, int size){	
		return new T[size]; // compile-time error
	}*/
	
	// ---------- 5 ----------------------
	// Cannot Create, Catch, or Throw Objects of Parameterized Types
	
	// ---------- 6 ----------------------
	// Same signature after type erasure.
	/*
	public class Example {
	    public void print(Set<String> strSet) { }
	    public void print(Set<Integer> intSet) { }
	}*/
	
	// --------- Even more ------------
	{ //No!, String gone
		//List<String>.class
	}
	
}
