package generic.typeparameter.bounded;

import generic.typeparameter.bounded.BoundedTypeparameterInterface.IMinMax;

/**
 * Possibilities for implementing a generic interface
 *
 * @author hajo
 *
 */
public class BoundedInterfaceVariations {

  // Possible but non-generic class (use Integer all over)
  class MyClass1 implements IMinMax<Integer> {

    int max, min;

    @Override
    public Integer max() {
      return max;
    }

    @Override
    public Integer min() {
      return min;
    }
  }

  // Silly  -------------------------------
  class MyClass2<T> implements IMinMax<Integer> {

    private T any;

    @Override
    public Integer max() {
      return (Integer) any;
    }

    @Override
    public Integer min() {
      return (Integer) any;
    }
  }

  // Try to make totally generic, but..-----------------------------------
  // Can't substitute T for T extends Comparable<T>
  /*
  class MyClass3<T> implements IMinMax<T> {

     T any;
     
     public T max() {
        return any;
     }

     public T min() {
        return any;
     }
  }
  */
  // --------------------------------------------------------
  // BAD, syntax error on extends, parameterization on class
  // Also type error strengthening T
  /*class MyClass4<T> implements IMinMax<T extends Comparable<T>> {  
     public T max() {
        return null;
     }

     public T min() {
        return null;
     }
  }*/
  

  // BAD, compiles but raw type (warnings)
  class MyClass5<T extends Comparable<T>> implements IMinMax {

    T any; // Hmm, what is this...

    @Override
    public T max() {
      return any;
    }

    @Override
    public T min() {
      return any;
    }
  }
  
  // Correct generic ---------------------------------------------
  class MyClass6<T extends Comparable<T>> implements IMinMax<T> {

    T any; 

    @Override
    public T max() {
      return any;
    }

    @Override
    public T min() {
      return any;
    }
  }
}
