package generic.typeparameter.bounded;

/**
 * Using and implementing generic interface with bounded type
 * @author hajo
 *
 */
public class BoundedTypeparameterInterface {

   public static void main(String[] args) {
      Integer inums[] = {
            3, 6, 2, 8, 6
      };
      Character chs[] = {
            'b', 'r', 'p', 'w'
      };

      // Integer and Character implements Comparable
      //MyClass<Object> i = new MyClass<Object>(inums);  // BAD Object not comparable
      
      ImplMinMax<Integer> i = new ImplMinMax<>(inums);
      System.out.println("Max value in inums: " + i.max());
      System.out.println("Min value in inums: " + i.min());
      
      ImplMinMax<Character> c = new ImplMinMax<>(chs);
      System.out.println("Max value in chs: " + c.max());
      System.out.println("Min value in chs: " + c.min());

      // Better (programming to an interface)
      IMinMax<Integer> mi = new ImplMinMax<>(inums);
      System.out.println("Max value in inums: " + mi.max());
      System.out.println("Min value in inums: " + mi.min());
   }
   
   // ------------ Types ------------------
   
   public interface IMinMax<T extends Comparable<T>> {

	   public T min();

	   public T max();
	}
   
   // Generic implementation of interface
   public static class ImplMinMax<T extends Comparable<T>> implements IMinMax<T> {

	   private T[] vals;

	   public ImplMinMax(T[] o) {
	      vals = o;
	   }

	   public T min() {
	      T v = vals[0];
	      for (int i = 1; i < vals.length; i++)
	         if (vals[i].compareTo(v) < 0)
	            v = vals[i];
	      return v;
	   }

	   public T max() {
	      T v = vals[0];
	      for (int i = 1; i < vals.length; i++)
	         if (vals[i].compareTo(v) > 0)
	            v = vals[i];
	      return v;
	   }
	}
   

}
