package generic.wildcard.bounded;

import java.util.Collections;
import java.util.List;

/**
 * Using lower bound wildcards
 * 
 * 
 * There's a simple rule, called the get-put principle, which tells us which 
 * kind of wildcard to use. The get-put principle, as stated in Naftalin
 * and Wadler's fine book on generics, Java Generics and Collections (see Resources), 
 * says:
 * 
 * Use an extends wildcard when you only get values out of a structure, use a super 
 * wildcard when you only put values into a structure, and don't use a wildcard 
 * when you do both.
 * 
 * The get-put principle is easiest to understand when applied to container classes 
 * like Box or the Collections classes, because the notion of getting or putting
 * connects naturally with what these classes do: store things. So, if we wanted
 * to apply the get-put principle to create a method that copies from one Box 
 * to another, the most general form would be as shown in Listing 8, where an 
 * upper-bounded wildcard is used for the source and a lower-bounded wildcard is 
 * used for the destination:
 * 
 * @author hajo
 *
 */
public class LowerBound {

    class A{}
    class B extends A{}
    class C extends B{}
    
    class D {}
    
    public static void main(String[] args) {
        List<A> al = null;
        List<B> bl = null; // Unused 
        List<C> cl = null;
        
        // This is a generic method
        // static <T> void copy(List<? super T> dest, List<? extends T> src)
        Collections.copy(al, cl);// type safe 
        
        // Explicit type variable (not necessary)
        Collections.<B>copy(bl, cl);
        
        // NO! 
        //Collections.copy(cl, bl);

    }

}
