package declarative;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

/**
 * 
 * Must be very careful with declarative style, risk for
 * StackOverflowError (try 10000 as in-param below)
 * Time seems to be a minor problem.
 * 

 * 
 * ASIDE: Tail recursive (java) functions are properly handled by some 
 * java JVMs (specifically the JITs) and not others!  
 * 
 * @author hajo
 * 
 */
public class DeclarativeEfficiency {

	public static void main(String[] args) {
		// A lot of tests
		long before = System.nanoTime();
		System.out.println(imperativeFactorial(2000));
		long after = System.nanoTime();
		System.out.println("Imperative : " + (after - before));
		
		before = System.nanoTime();
		System.out.println(declarativeFactorial(BigInteger.valueOf(2000)));
		after = System.nanoTime();
		System.out.println("Declarative : " + (after - before));

		
		before = System.nanoTime();
		System.out.println( declarativeSum(2000));
		after = System.nanoTime();
		System.out.println("Declarative : " + (after - before));
		before = System.nanoTime();
		System.out.println( imperativeSum(2000));
		after = System.nanoTime();
		System.out.println("Imperative : " + (after - before));
	}
		
	// Declarative version (using recursion)
	public static int declarativeSum (int n){
		if( n == 0){
			return 0;
		}else{
			return n + declarativeSum(n-1);
		}
	}
	// Imperative version (manipulating state)
	public static int imperativeSum (int n){
		int sum = 0;
		for( int i = 1 ; i <= n ; i++ ){
			sum += i;
		}
		return sum;
	}
	
	// Declarative version (using recursion) 
	public static BigInteger declarativeFactorial(BigInteger i) {
		if (i.equals(BigInteger.valueOf(1))) {
			return BigInteger.valueOf(1);
		} else {
			return i.multiply(declarativeFactorial(i.subtract(BigInteger
					.valueOf(1))));
		}
	}

	// Imperative version (manipulating state)
	public static BigInteger imperativeFactorial(int n) {
		BigInteger result = BigInteger.valueOf(1);
		for (int i = 2; i <= n; i++) {
			result = result.multiply(BigInteger.valueOf(i));
		}
		return result;
	}

	// ------ NOT USED Fibonacci using Memoization ------------------------
	
	private static List<BigInteger> fibCache = new ArrayList<BigInteger>();
	static {
	      fibCache.add(BigInteger.ZERO);
	      fibCache.add(BigInteger.ONE);
	}
	
	 public static BigInteger fibMemoized(int n) {
         if (n >= fibCache.size()) {
             fibCache.add(n, fibMemoized(n-1).add(fibMemoized(n-2)));
         }
         return fibCache.get(n);
  }

}
