package sideeffect;

import java.util.Arrays;


/**
 * 
 * In an effort to clarify side-effects we partition methods into two distinct
 * categories:
 * 
 * Mutators: methods that change state.
 * 
 * Accessors: Methods (functions) that access the state (read state)
 * 
 * See comments
 * 
 * @author hajo
 * 
 */
public class AccessorMutator {

	public static void main(String[] args) {
		MyClass c = new MyClass();
		c.push(1);
		c.push(2);
		c.push(3);
		System.out.println(c);
		
		System.out.println("Top is " + c.top());
		c.pop();
		System.out.println(c);
		System.out.println("Top is " + c.top());
			
	}

	// No exception handling
	public static class MyClass {

		private int[] elems; // Store some int's
		private int sp;

		public MyClass() { 
			final int DEFAULT = 8;
			elems = new int[DEFAULT];
			sp = -1;
		}

		// Mutator
		public void push(int i) { 
			elems[++sp] = i; // Move sp and add element and move 
		}

		// Mutator
		public int pop() {  // Remove element at sp
			int i = elems[sp];
			elems[sp--] = 0;
			return i;            
		}

		// Accessor
		public int top() {
			return elems[sp];  // Inspect value at sp
		}

		// Accessor
		@Override
		public String toString() {
			return "MyClass [elems=" + Arrays.toString(elems) + ", sp=" + sp
					+ "]";
		}	
	}
}
