package edu.gu.cid.dict.io;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;


/**
 * Simple reader for row based textual data. Should be in some IO project.
 * 
 * File format for data;
 * 
 * Row Row Row ...
 * 
 * Where row is:
 * chars EOL  (EOL system dependent end of line handled by Scanner)
 * 
 * Rows starting with # are comments
 * 
 * @author hajo
 * 
 */
public class RowReader {

    public static final char DEFAULT_COMMENT_CHAR = '#';
	private char commentChar = DEFAULT_COMMENT_CHAR;
	private boolean skipEmptyLines = true;
  
    
    public void setCommentChar(char ch){
    	commentChar = ch;
    }
    
    public void skipEmtpyLines(boolean b){
    	skipEmptyLines = b;
    }
     
	public List<String> read(URI uri)
            throws IOException, URISyntaxException {
        Scanner sc = null;
        List<String> entries = new ArrayList<>();
        if ("file".equals(uri.getScheme())) {
            sc = new Scanner(new File(uri));
        } else if ("http".equals(uri.getScheme())) {
            sc = new Scanner(uri.toURL().openStream());
        } else {
            throw new IllegalArgumentException("Bad scheme");
        }
        try {
            String line = null;
            while (sc.hasNextLine()) {
                line = sc.nextLine();
                if ( !isEmptyLine(line) && !isCommentLine(line)) {
                    entries.add(line.trim());
                }
            }
        } finally {
            sc.close();
        }
        return entries;
    }
	
	private boolean isEmptyLine(String line){
		return skipEmptyLines && line.length() == 0;
	}
	
	private boolean isCommentLine(String line){
		return line.charAt(0) == commentChar;
	}
	
    
}
