package edu.gu.cid.translator.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import edu.gu.cid.translator.core.Language;
import edu.gu.cid.translator.ctrl.LanguageCtrl;
import edu.gu.cid.translator.event.Event;
import edu.gu.cid.translator.event.IEventHandler;
import edu.gu.cid.translator.util.SelectItem;

/**
 * Panel for the combo boxes to select from and to language.
 * 
 * @author hajo
 * 
 */
@SuppressWarnings("serial")
public class LanguageSelectionPanel extends JPanel implements ActionListener,
		IEventHandler {

	private final JComboBox<SelectItem> fromBox = new JComboBox<>();
	private final JComboBox<SelectItem> toBox = new JComboBox<>();
	private final JLabel fromLabel = new JLabel("from");
	private final JLabel toLabel = new JLabel("to");
	

	public LanguageSelectionPanel(LanguageCtrl languageCtrl) {
		init();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		
	}

	@Override
	public void onEvent(Event evt) {
		
		validate();
	}

	private void setSelected(JComboBox<SelectItem> cBox, Language lang) {
		// Else the combo box will fire
		cBox.removeActionListener(this);
		for (int i = 0; i < cBox.getItemCount(); i++) {
			SelectItem si = cBox.getItemAt(i);
			if (si.getValue().equals(lang)) {
				cBox.setSelectedIndex(i);
				// Put back listener
				cBox.addActionListener(this);
				break;
			}
		}
	}

	private void init() {
		SelectItem[] selections = getSelectItems();
		fromBox.setModel(new DefaultComboBoxModel<SelectItem>(selections));
		toBox.setModel(new DefaultComboBoxModel<SelectItem>(selections));
		fromBox.addActionListener(this);
		toBox.addActionListener(this);
		this.add(fromLabel);
		this.add(fromBox);
		this.add(toLabel);
		this.add(toBox);
	}

	// Hard coded should be from config files
	private SelectItem[] getSelectItems() {
		SelectItem[] selections = new SelectItem[] {
				new SelectItem("Select", new Language(Language.Tag.xx_XX)),
				new SelectItem("English", new Language(Language.Tag.en_US)),
				new SelectItem("Swedish", new Language(Language.Tag.sv_SV)) };
		return selections;
	}

}
