package edu.gu.cid.translator.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import edu.gu.cid.translator.ctrl.ExitCtrl;
import edu.gu.cid.translator.ctrl.StartCtrl;



/**
 * Main frame for application. This is just a container for panels
 * and a menu.
 * @author hajo
 *
 */
@SuppressWarnings("serial")
public class MainFrame extends JFrame {
	
	private final StartCtrl startCtrl;
	private final ExitCtrl exitCtrl;

	public MainFrame(InputPanel controlPanel, DisplayPanel ioPanel,
			LanguageSelectionPanel languageSelectionPanel, StartCtrl startCtrl,
			ExitCtrl exitCtrl) {
		this.startCtrl = startCtrl;
		this.exitCtrl = exitCtrl;
		this.setLayout(new BorderLayout());
		this.addWindowListener(listener);
		this.getContentPane().add(controlPanel, BorderLayout.SOUTH);
		this.getContentPane().add(ioPanel, BorderLayout.CENTER);
		this.getContentPane().add(languageSelectionPanel, BorderLayout.NORTH);
		initMenu();
		pack();
	}

	// Common listener for menu
	private ActionListener menuListener = new ActionListener() {

		@Override
		public void actionPerformed(ActionEvent e) {
			Object s = e.getSource();
			if (s == file_exit) {
				exitCtrl.doExit();
			} else if (s == about_about) {
				new AboutDialog(MainFrame.this).setVisible(true);
			}
		}
	};

	private final WindowListener listener = new WindowAdapter() {
		@Override
		public void windowClosing(WindowEvent e) {
			exitCtrl.doExit();
		}

		@Override
		public void windowOpened(WindowEvent e) {
			startCtrl.doStart();
		}
	};

	// ----- The menu ---------

	private JMenuBar menuBar = new JMenuBar();
	private JMenu file = new JMenu("File");
	private JMenu about = new JMenu("About");
	private JMenuItem file_exit = new JMenuItem("Exit");
	private JMenuItem about_about = new JMenuItem("About");

	private void initMenu() {
		this.setJMenuBar(menuBar);
		menuBar.add(file);
		menuBar.add(about);
		file.add(file_exit);	
		about.add(about_about);	
		file_exit.addActionListener(menuListener);
		about_about.addActionListener(menuListener);
	}
}
