package threads.deadlock.callingout;

import java.util.LinkedList;
import java.util.List;

public class WorkQueue {

	// LinkedList NOT synchronized
	private final List<String> queue = new LinkedList<String>();
	private boolean stopped = false;

	public WorkQueue() {
		new WorkerThread().start();
	}

	// Inner class
	public class WorkerThread extends Thread {

		public WorkerThread() {
			super("WorkerThread");
		}

		public void run() {
			String workItem = null;
			while (true) {
				synchronized (queue) {
					try {
						while (queue.isEmpty() && !stopped) {
							System.out.println("Queue wait...");
							queue.wait();
						}
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
					if (stopped) {
						return;
					}

					workItem = queue.remove(0);
					System.out.println(Thread.currentThread().getName()
							+ " removes..." + workItem);
					//} //
					try {
						// Calling out from synchronized block (or NOT)
						processItem(workItem);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				} //
			}
		}
	}

	// Silly to start thread here but this is just a demonstration
	public void processItem(final String workItem) throws InterruptedException {

		// Current thread is WorkerThread

		Thread child = new Thread() {

			public void run() {
				System.out.println(Thread.currentThread().getName()
						+ " Child enqueue..." + workItem);
				// queue locked
				enqueue(workItem);
			}
		};
		child.start();

		// WorkerThread should wait for child to finish.
		// But that never happens...
		child.join();
	}

	public void enqueue(String workItem) {
		synchronized (queue) {
			queue.add(workItem);
			queue.notifyAll();
		}
	}
}
