package rmi;

import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.Scanner;

/**
 * Start server application
 * @author hajo
 */
public class ServerMain {

    public static final String SERVER_NAME = "server";
    public static final int SERVER_PORT = 9999;
    public static final int REGISTRY_PORT = 8888;

    public static void main(String args[]) {
        try {
            /*
             * Create remote object
             */
            final Server server = new Server();
            /*
             * All remote objects must be exported to the RMI
             * subsystem. Returnvalue is a stub (i.e. a proxy for the server, 
             * see Proxy design pattern
             */
            IServer stub = (IServer) UnicastRemoteObject.exportObject(server,
                    SERVER_PORT);
            /*
             * Create a registry
             */
            LocateRegistry.createRegistry(REGISTRY_PORT);
            Registry registry = LocateRegistry.getRegistry(REGISTRY_PORT);
            /*
             * Bind the stub in the registry (so clients can find it)
             */
            registry.rebind(SERVER_NAME, stub);
            System.out.println("Server stub bound in registry at port "
                    + REGISTRY_PORT);

            System.out.println("Server running at (localhost) 127.0.0.1:"
                    + SERVER_PORT);
            System.out.println("bye to quit ");

        } catch (Exception e) {
            System.out.println("Server exception: " + e.getMessage());
            e.printStackTrace();
        }

        // Command line
        Scanner scanner = new Scanner(System.in);
        while (true) {
            System.out.print("> ");
            String cmd = scanner.nextLine();
            if ("bye".equals(cmd)) {
                scanner.close();
                System.exit(0);
            }
        }

    }
}
