package freeze;


/**
 * Possible way to make an freezable class i.e. initially mutable 
 * but later immutable 
 * 
 * 
 * @author hajo
 * 
 */
public final class FreezableCreator {

	private Freezable f = new Freezable();

	public FreezableCreator setS(String s) {
		// No setter but f.s visible here
		f.s = s;
		return this;
	}

	public FreezableCreator setO(Object o) {
		f.o = o;
		return this;
	}

	public Freezable freeze() {
		if (f.o == null || f.s == null) {
			throw new IllegalArgumentException("Must initialize");
		}
		Freezable f = this.f;
		this.f = null;
		return f;
	}

	// Inner class to be created
	public static final class Freezable {
		private String s;
		private Object o;

		// Assume must have
		public Freezable() {
		}

		public String getS() {
			return s;
		}

		public Object getO() {
			return o;
		}
	}

}
