package immutable;

/**
 * This show that during construction (in constructor)
 * an immutable class is mutable
 * 
 * Really avoid letting "this" leak from constructor
 * 
 * @author hajo
 *
 */
public class LeakingThis {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		EventHandler e = new EventHandler();

		UnsafeWindow uw = new UnsafeWindow(e, 1, 2);

		System.out.println(uw.getX() + uw.getY()); // 3 as suspected
		System.out.println(e.getImportantStuff()); // 0 ????

	}

	// Class is immutable!
	public static final class UnsafeWindow {

		final private int x, y;

		// Here the object is mutable 
		public UnsafeWindow(EventHandler e, int argx, int argy) {
			e.register(this); // Don't do! Leaking this
			x = argx;
			y = argy;
		}

		public int getX() {
			return x;
		}

		public int getY() {
			return y;
		}

	}

	public static class EventHandler {

		private UnsafeWindow uw;
		private int importantStuff;

		public void register(UnsafeWindow uw) {
			this.uw = uw;
			doImportantStuff();
		}

		private void doImportantStuff() {
			// This is a "call back" to UnsafeWindow
			int i = uw.getX(); // x not assigned, default to 0
			int j = uw.getY();
			importantStuff = i + j;
		}

		public int getImportantStuff() {
			return importantStuff;
		}
	}
}
