package outparams;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 * Using out parameters to avoid state
 * @author hajo
 *
 */
public class UsingOutParams {

	public void getAllNodes(final List<Node> nodes) {
		getAllNodes( this.root, nodes);
	}
	
	/*
	 * Recursion using state, bad...
	 */
	private List<Node> nodes = new ArrayList<Node>();

	private void getAllNodes(Node n) {
		if (n == null) {
			return;
		} else {
			nodes.add(n);
			getAllNodes(n.getLeft());
			getAllNodes(n.getRight()); 
		}
	}


	/*
	 * Better, "stateless" recursion, but using output parameters 
	 */
	// Mark with final as i hint, this is special
	private void getAllNodes(Node n, final List<Node> nodes) { // <---out param
		if (n == null) {
			return;
		} else {
			nodes.add(n);
			getAllNodes(n.getLeft(), nodes);
			getAllNodes(n.getRight(), nodes);
		}
	}
	
	
	// ---------- Building the tree -----------------------------

	public Node root;

	public UsingOutParams() {
		root = buildStructure();
	}

	private Node buildStructure() {
		// Rude
		return new Node(new Node(new Node(null, null), new Node(new Node(null,
				null), null)), new Node(new Node(null, new Node(null, null)),
				new Node(null, null)));

	}
	
	// ------- TYPES ---------------------
	
	public static class Node {
		private static Random r = new Random();
		private int value = r.nextInt(3);
		private Node left;
		private Node right;

		public Node(Node left, Node right){ 
			this.left = left;
			this.right = right;
		}
		
		public Node getLeft() {
			return left;
		}
		public Node getRight() {
			return right;
		}
		public int getValue() {
			return value;
		}

		@Override
		public String toString() {
			return "[" + value + "]";
		}
	}
	
	
	
}
