package freezable;

/**
 * Possible way to make an freezable class i.e. initially mutable but later
 * immutable
 *
 * NOTE: Outer class will be frozen object
 *
 * @author hajo
 *
 */
public final class Freezable {

    private String s;
    private Object o;

    // Assume must have (so can't use final for attributes)
    public Freezable() {
    }

    public String getS() {
        // Possible exception if null
        return s;
    }

    public Object getO() {
        // Possible exception if null
        return o;
    }

    // Inner creator class
    public static class New {

        private Freezable f = new Freezable();

        public New setS(String s) {
            // No set method on f but we're inside Freezable
            f.s = s;
            return this;
        }

        public New setO(Object o) {
            f.o = o;
            return this;
        }

        public Freezable freeze() {
            if (this.f == null) {
                throw new IllegalStateException("Object already frozen");
            }
            if (f.o == null || f.s == null) {
                throw new IllegalArgumentException("Object not initialize");
            }

            Freezable f = this.f;
            this.f = null;
            return f;
        }

    }

}
