package immutable;

/**
 * Here we have a reference to a mutable object! 
 * Have an incoming shared reference, must copy!
 * 
 * @author hajo
 * 
 */
public class DefensiveCopyIn {

	public static void main(String[] args) {
		// Bad attempt
		StringBuffer b = new StringBuffer();
		b.append("abc");
		MyLeakingClass m = new MyLeakingClass(b);
		b.append("OhOh");

		System.out.println(m.getData()); // State changed

		// Correct attempt
		b = new StringBuffer();
		b.append("abc");
		MyImmutableClass i = new MyImmutableClass(b);
		b.append("OhOh");
		System.out.println(i.data); // State **unchanged**
	}

	// An incorrect attempt to create an immutable class
	public static final class MyLeakingClass {
		// reference final, but object **NOT** immutable
		private final StringBuffer data;

		// Incoming reference is shared!
		public MyLeakingClass(StringBuffer b) {
			this.data = b;
		}

		// Correct returning defensive copy
		public StringBuffer getData() {
			return new StringBuffer(data);
		}
	}

	// A correct attempt to create an immutable class
	public static final class MyImmutableClass {
		// reference final, but object **NOT** immutable
		private final StringBuffer data;

		// Defensive copy of incoming reference
		public MyImmutableClass(StringBuffer b) {
			data = new StringBuffer(b);
		}

		// Correct returning defensive copy
		public StringBuffer getData() {
			return new StringBuffer(data);
		}
	}

}
