package immutable;

/**
 * Sometimes immutable objects are to costly to use
 * Example shows String (immutable) vs StringBuilder (mutable)
 * @author hajo
 *
 */
public class Efficiency {

	public static void main(String[] args) {
		StringProducer sp = new StringProducer();
	
		// Very slow
		long before = System.currentTimeMillis();
		sp.horrible();
		long after = System.currentTimeMillis();
		long t1 = after - before;

		// Quick
		before = System.currentTimeMillis();
		sp.theWayToDoIt();
		after = System.currentTimeMillis();
		long t2 = after - before;

		System.out.println("Tid: String = " + t1 + " StringBuilder = " + t2);
	}

	public static class StringProducer {
		private final int manyMany = 1000;

		// String is immutable, new objects
		// created and values (chars) copied from old
		// DON't do this
		public String horrible() {
			String s = "";
			for (int i = 0; i < manyMany; i++) {
				s += getLongString(); // + Very slow
			}
			return s;
		}

		// Much quicker
		public String theWayToDoIt() {
			StringBuilder b = new StringBuilder(manyMany * 30);
			for (int i = 0; i < manyMany; i++) {
				b.append(getLongString());
			}
			return b.toString(); // Return immutable
		}

		private String getLongString() {
			return "qwertyuiopasdfghjklmnzxcvbnm";
		}
	}
}
