package statics;

/**
 * Other usage for purely static (stateless) classes; Factories i.e.
 * a class responsible for creating other (possible complex) objects.
 * 
 * From book: Effective Java sec ed.
 * 
 * One advantage of static factory methods is that, unlike constructors, they have names
 * (can choose name, constructors must be named...)
 * 
 * A second advantage of static factory methods is that, unlike constructors, 
 * they are not required to create a new object each time they’re invoked.
 * (could save and return same)
 * 
 * A third advantage of static factory methods is that, unlike constructors, 
 * they can return an object of any subtype of their return type.
 * 
 * A fourth advantage of static factory methods is that they reduce
 * the verbosity of creating parameterized type instances (more later)
 * 
 * 
 * @author hajo
 *
 */
public final class StaticFactory {
	// Can't create objects
	private StaticFactory(){	
	}
	
	/* 
	 * The FACTORY method
         *  Avoid new in code, possible return
         *  a subtype
	 */
	public static MyClass newMyClass(){
                // Could have a more complicated construction process here
		return new MyClass();
	}
	
	// --------- TYPES --------------
	
	// Class to be manufactured (must be static here, don't bother for now)
	public static class MyClass{
	}
	
}
