package instanceofgetClass;

/**
 * Demonstration of runtime type information
 * I.e. there is type information to use during runtime 
 * (by the system)
 * 
 * This is special, avoid explicit use if possible
 * 
 * @author hajo
 *
 */
public class RTTI {
	
	public static void main(String[] args) {
		IMyInterface mi1 = new A(); // Type compatible
		IMyInterface mi2 = new B(); // Type compatible
		
		// Get object representing the class (need an object)
		Class<? extends IMyInterface> clazz1 = mi1.getClass();
		// Will print the runtime type
		System.out.println(clazz1.getSimpleName());
		System.out.println(clazz1.getInterfaces()[0].getSimpleName());
		System.out.println(clazz1.getSuperclass().getSimpleName());
		
		Class<? extends IMyInterface> clazz2 = mi2.getClass();
		// Will print the runtime type
		System.out.println(clazz2.getSimpleName());
		System.out.println(clazz1.getInterfaces()[0].getSimpleName());
		System.out.println(clazz2.getSuperclass().getSimpleName());
		
		// Other way to access the Class<?> object (need class only)
		Class<? extends IMyInterface> clazz3 = A.class;
		Class<? extends IMyInterface> clazz4 = B.class;
		System.out.println(clazz3.getSimpleName());
		System.out.println(clazz4.getSimpleName());
		System.out.println(IMyInterface.class.getSimpleName());
		
		/*
	    At run time, the result of the instanceof operator is true if the 
	    value of the RelationalExpression is not null and the reference 
	    could be cast (JLS 15.16) to the ReferenceType without raising a 
	    ClassCastException. Otherwise the result is false.
	    */
		
		// Is mi1 type compatible with A (or sub type)
		if( mi1 instanceof A){
			System.out.println("Yes it's A");
			A a = (A) mi1; // Safe
		}
		if( !(mi1 instanceof B)){
			System.out.println("No, it's not B");
		}
		if( mi1 instanceof IMyInterface){
			System.out.println("Yes it's IMyInterface");
		}
		
		 // Always false
	    System.out.println(null instanceof Object);
	    
	    //No allowed, silly 
	    //System.out.println(b instanceof ImplA);

	    // Order matters, object first!
	    //System.out.println(IA instanceof ia);
	    
	}
	
	// ------------------ TYPES ---------------------------

	public static interface IMyInterface {
		public void doit();
	}
	public static class A implements IMyInterface {
		@Override
		public void doit(){
			System.out.println("A");
		}
	}
	public static class B implements IMyInterface {
		@Override
		public void doit(){
			System.out.println("B");
		}
	}
	
	
}
