package nulls;

import java.util.ArrayList;
import java.util.List;

/**
 * Don't allow storing of nulls (or possible
 * forward nulls to others...
 * @author hajo
 *
 */
public class DontStoreNulls {

	public static void main(String[] args) {
		// Bad style
		MyBadContainer m = new MyBadContainer();
		m.add(null);
		String s = m.get(0);

		s.length(); // Disaster!!!

	}

	// DONT return nulls
	public static class MyBadContainer {

		List<String> strs = new ArrayList<>();

		public void add(String val) {
			// if val is null?
			strs.add(val);
		}

		public String get(int index) {
			return strs.get(index);
		}
	}

	// Better way to do
	public static class MySmartContainer {

		List<String> strs = new ArrayList<>();

		public boolean add(String val) {
		    if( val == null ){
		        throw new NullPointerException("nulls not allowed");
		    }
			return strs.add(val);
		}

		public String get(int index) {
			return strs.get(index);
		}
	}

}
